import wave
from io import BytesIO


class ToData:
    def __init__(self, frames, params, lib="wave"):
        self.frames = frames
        self.params = params
        self.lib = lib

        self.libs = {
            "wave": self.wave,
        }

        self.data = self.libs[self.lib]()

    def wave(self):
        with BytesIO() as f:
            with wave.open(f, "wb") as wav:
                wav.setparams(self.params)

                for i in range(self.params[3]):
                    wav.writeframes(bytes(self.frames[i]))

            f.seek(0)
            data = f.read()

        return data

    def __bytes__(self):
        return self.data



try:
    from mpmath import log
except ImportError:
    from math import log


class Frame:
    def __init__(self, value):
        if isinstance(value, int):
            self.value = int.to_bytes(
                value, length=self.bytes_needed(value), byteorder="little"
            )

        elif isinstance(value, bytes):
            self.value = value

            # if len(value) == 2:
            # self.value = value

            # else:
            # raise NameError

    def __int__(self):
        return int.from_bytes(self.value, byteorder="little")

    def __str__(self):
        return str(self.__int__())

    def __bytes__(self):
        return self.value

    def __repr__(self):
        return f"Frame({self.__str__()})"

    def __reversed__(self):
        pass

    def __add__(self, other):
        from .audio import Audio

        if isinstance(other, Frame):
            return Audio(frames=[self, other])
        if isinstance(other, Audio):
            return Audio(frames=[self, *other.frames])

    def __mul__(self, other):
        from .audio import Audio

        if isinstance(other, int):
            return Audio(frames=[self] * other)
        else:
            raise TypeError(
                f"can't multiply sequence by non-int of type '{type(other).__name__}'"
            )

    def __gt__(self, other):
        return self.value > other.value

    def bytes_needed(self, n):
        if n == 0:
            return 1
        return int(log(n, 256)) + 1



from .frame import Frame
from .play import Play
from .libdata import ToData
import platform
import wave
import subprocess
import copy

import pathlib
import io

lib = "wave"


def get_os():
    os = platform.system()

    if os == "Linux":
        if subprocess.check_output(["uname", "-o"]).strip() == b"Android":
            return "termux"
        else:
            return "linux"
    else:
        return os.lower()


class Audio:
    def __init__(self, *args, **kwargs):
        self.frames = (
            list(args[0])
            if len(args) == 1 and isinstance(args[0], (list, tuple))
            else list(args)
        )

        if "frames" in kwargs:
            self.frames.extend(kwargs["frames"])

        if "params" in kwargs:
            parameters = kwargs["params"]
            self.channels, self.frame_rate, = (
                parameters[0],
                parameters[2],
            )

        else:
            self.frame_rate = kwargs.setdefault("frame_rate", 44.1 * 1000)
            self.channels = kwargs.setdefault("channels", 1)

    @classmethod
    def open(cls, file):
        if isinstance(file, pathlib.Path):
            file = io.FileIO(file, "rb")

        if isinstance(file, (str, pathlib.Path, io.IOBase)):
            with wave.open(file, "rb") as f:
                sample_width = f.getsampwidth()
                frame_rate = f.getframerate()
                frame_count = f.getnframes()
                channels = f.getnchannels()

                frames = [Frame(f.readframes(1)) for i in range(frame_count)]

        return cls(
            sample_width=sample_width,
            frame_rate=frame_rate,
            channels=channels,
            frames=frames,
            frame_count=frame_count,
        )

    @property
    def duration(self):
        return self.frame_count / self.frame_rate

    @property
    def frame_width(self):
        return self.channels * self.sample_width

    @property
    def sample_width(self):
        default = 16
        if self.frames:
            return len(bytes(self.max()))
        else:
            return default

    @property
    def frame_count(self):
        return len(self.frames)

    @property
    def params(self):
        return (
            self.channels,
            self.sample_width,
            self.frame_rate,
            self.frame_count,
            "NONE",
            "Uncompressed",
        )

    def play(self):
        # data = self.frames_to_data(self.frames)

        Play(
            get_os(),
            ToData(self.frames, self.params, lib).data,
        )

    def save(self, file):
        if isinstance(file, str):
            file = open(file, "wb")
        with file as f:
            f.write(ToData(self.frames, self.params, lib).data)

    def append(self, frame):
        self.frames.append(frame)

    def extend(self, frames):
        self.frames.extend(frames)

    def max(self):
        return max(self.frames)

    def __len__(self):
        return self.frame_count

    def __getitem__(self, index):
        if isinstance(index, slice):
            return Audio(self.frames[index], params=self.params)
        elif isinstance(index, int):
            return self.frames[index]
        else:
            raise TypeError(
                f"Audio indices must be integers or slices, not {type(index)}"
            )

    def __setitem__(self, key, value):
        if isinstance(key, (slice, int)):
            if isinstance(value, Frame):
                self.frames[key] = value
            else:
                raise TypeError("Audio value must be Frame")
        else:
            raise TypeError(
                f"Audio indices must be integers or slices, not {type(key)}"
            )

    def __add__(self, other):
        frames = copy.deepcopy(self.frames)
        if isinstance(other, Frame):
            frames.append(other)
        elif isinstance(other, Audio):
            frames.extend(other.frames)

        return Audio(frames=frames)

    def __mul__(self, other):
        frames = copy.deepcopy(self.frames)
        if isinstance(other, int):
            frames *= other
        else:
            raise TypeError(
                f"can't multiply sequence by non-int of type '{type(other).__name__}'"
            )

        return Audio(frames=frames)

    def __gt__(self, other):
        return self.frames > other.frames



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['paud']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'paud',
    'version': '0.1',
    'description': 'Python library to process audio',
    'long_description': "# paud\nPython library to process audio\n\n\n\nopen audio\n\n```python\nfrom paud import Audio\n\nau = Audio.open('audio.wav') #file or path\n\nprint(f'audio duration = {au.duration}')\n\nau.play() #play audio\n\nau[:100].play() #play first 100 frames of audio\n\n\n```\n\n\n\nopen audio from URL\n\n```python\nimport requests\nfrom paud import Audio\nfrom io import BytesIO\n\nr = requests.get('https://file-examples-com.github.io/uploads/2017/11/file_example_WAV_10MG.wav')\n\nau = Audio.open(BytesIO(r.content))\nau.play()\n```\n\n",
    'author': 'ASVIEST',
    'author_email': '71895914+ASVIEST@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ASVIEST/paud',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import wave
import tempfile
import soundfile as sf
from io import BytesIO


class Play:
    # enviroments = {
    #     'termux': termux,
    #     'windows': windows
    # }

    def __init__(self, env, data):
        self.env = env
        self.data = data

        self.enviroments = {
            "termux": self.termux,
            "windows": self.windows,
            "linux": self.linux,
            "macos": self.macos,
        }

        self.enviroments[self.env]()

    def termux(self):
        from termux import Media

        with tempfile.NamedTemporaryFile() as temp:
            temp.write(self.data)
            Media.play(temp.name)

    def windows(self):
        import winsound

        winsound.PlaySound(self.data, winsound.SND_MEMORY | winsound.SND_NOSTOP)

    def linux(self):
        pass

    def macos(self):
        pass

    def pyaudio_play(self):
        pass
